--[[]
-------------------------------------------------------------------------------
-- CyroSiege, by Ayantir
-------------------------------------------------------------------------------
This software is under : CreativeCommons CC BY-NC-SA 4.0
Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)

You are free to:

    Share — copy and redistribute the material in any medium or format
    Adapt — remix, transform, and build upon the material
    The licensor cannot revoke these freedoms as long as you follow the license terms.


Under the following terms:

    Attribution — You must give appropriate credit, provide a link to the license, and indicate if changes were made. You may do so in any reasonable manner, but not in any way that suggests the licensor endorses you or your use.
    NonCommercial — You may not use the material for commercial purposes.
    ShareAlike — If you remix, transform, or build upon the material, you must distribute your contributions under the same license as the original.
    No additional restrictions — You may not apply legal terms or technological measures that legally restrict others from doing anything the license permits.


Please read full licence at : 
http://creativecommons.org/licenses/by-nc-sa/4.0/legalcode
]]


-- Init CyroSiege variables
local ADDON_NAME = "CyroSiege"
local ADDON_VERSION = "4"
local ADDON_AUTHOR = "Ayantir"
local ADDON_WEBSITE = "http://www.esoui.com/downloads/info703-CyroSiege.html"
local db

-- Returns true if consumable, needed to check if stack is stackable
local function IsSiegeConsumable(itemId)
	if itemId == "27138" or itemId == "27962" then
		return true
	else
		return false
	end
end

-- Build an array of itemID's faction dependant for the 15 items to check
local function FactionConversion()

	-- Get player faction for faction items
	local playerfaction = GetUnitAlliance("player")
	local factionTable = {}
	
	--Common items
	factionTable[1] = "30359" --Flaming Oil
	-- Consumables
	factionTable[2] = "27138" --Keep Wall Masonry Repair Kit
	factionTable[3] = "27962" --Keep Door Woodwork Repair Kit
	
	-- Siege items, factio dependant and not stackable
	if playerfaction == ALLIANCE_ALDMERI_DOMINION then
		factionTable[4] = "29533" --Forward Camp
		factionTable[5] = "27136" --Battering Ram
		factionTable[6] = "36567" --Ballista
		factionTable[7] = "27970" --Fire Ballista
		factionTable[8] = "27973" --Lightning Ballista
		factionTable[9] = "27105" --Firepot Trebuchet
		factionTable[10] = "44768" --Iceball Trebuchet
		factionTable[11] = "44769" --Stone Trebuchet
		factionTable[12] = "27964" --Meatbag Catapult
		factionTable[13] = "27967" --Oil Catapult
		factionTable[14] = "44770" --Scattershot Catapult	
	elseif playerfaction == ALLIANCE_EBONHEART_PACT then
		factionTable[4] = "29534"
		factionTable[5] = "27850"
		factionTable[6] = "36568"
		factionTable[7] = "27971"
		factionTable[8] = "27974"
		factionTable[9] = "27114"
		factionTable[10] = "44775"
		factionTable[11] = "44776"
		factionTable[12] = "27965"
		factionTable[13] = "27968"
		factionTable[14] = "44777"
	elseif playerfaction == ALLIANCE_DAGGERFALL_COVENANT then
		factionTable[4] = "29535"
		factionTable[5] = "27835"
		factionTable[6] = "36569"
		factionTable[7] = "27972"
		factionTable[8] = "27975"
		factionTable[9] = "27115"
		factionTable[10] = "44771"
		factionTable[11] = "44772"
		factionTable[12] = "27966"
		factionTable[13] = "27969"
		factionTable[14] = "44773"
	end
   
   factionTable[15] = "27112" --Siege Repair Kit
	factionTable[16] = "33271" --Grand Soul Gem
	factionTable[17] = "71073" --Battle Draught
	factionTable[18] = "71071" --Health Draught
	factionTable[19] = "71072" --Spell Draught
   
	return factionTable
	
end

local function GetItemLinkItemId(itemLink)
	return select(4, ZO_LinkHandler_ParseLink(itemLink))
end

-- Replace faction name with the good one
local function factionItemsNames(itemName)
	
	-- Get player faction for faction items
	local playerfaction = GetUnitAlliance("player")
	
	-- Siege items, faction dependant and not stackable
	-- ALLIANCE_DAGGRTFALL_COVENANT = Nothing to do, strings are already Covenant ones
	if playerfaction == ALLIANCE_ALDMERI_DOMINION then
		-- Dominion
		--FR
		itemName = string.gsub(itemName, "de l'Alliance", "du Domaine")
		--EN
		itemName = string.gsub(itemName, "Covenant", "Dominion")
		--DE
		itemName = string.gsub(itemName, "Bündnis", "Dominion")
	elseif playerfaction == ALLIANCE_EBONHEART_PACT then
		--Pact
		itemName = string.gsub(itemName, "de l'Alliance", "du Pacte")
		itemName = string.gsub(itemName, "Covenant", "Pact")
		itemName = string.gsub(itemName, "Bündnis", "Pakt")
	end
	
	return itemName
	
end

-- shoplist array[ItemID] with value = How much to need to buy compared to your neededlist
-- alreadygot array[ItemID] with value = How much I already got in bacgpack
local function GetShoppingList()

	--Inits
	local neededList = {}
	local shopList = {}
	local alreadygot = {}
	local goldList = {}
	
	-- Transforms id 1 into itemID item, with faction conversion bundled
	local factionTable = FactionConversion()
	
	-- For each item , get the needed qty to get
	for siegeItemId, mintoget in pairs(db.shoppinglist) do
		if tonumber(mintoget) > 0 then
			neededList[factionTable[siegeItemId]] = tonumber(mintoget)
			goldList[factionTable[siegeItemId]] = db.goldlist[siegeItemId]
		end
	end
	
	-- Check bagpack
	for slotId = 0, GetBagSize(BAG_BACKPACK) do
		
		-- get ID of item
		local bagItemId = GetItemLinkItemId(GetItemLink(BAG_BACKPACK, slotId))
		
		-- If item is siege and needed and already present in bagpack
		if neededList[bagItemId] then
		
			-- Stack already in bagpack, size can be 1 or more
			local stackSize = GetSlotStackSize(BAG_BACKPACK, slotId)
		
			-- Get qty of items in whole bag
			if alreadygot[bagItemId] then
				alreadygot[bagItemId] = alreadygot[bagItemId] + stackSize
			else
				alreadygot[bagItemId] = stackSize
			end
			
		end
	end
	
	-- For each item we need with qty > 0
	for siegeItemId, mintoget in pairs(neededList) do
		-- If I already get some
		if alreadygot[siegeItemId] then
			-- If i get less than needed
			if alreadygot[siegeItemId] < tonumber(mintoget) then
				-- Buy the difference
				shopList[siegeItemId] = tonumber(mintoget) - alreadygot[siegeItemId]
			else
				-- Nothing to buy
			end
		else
			-- Don't got any item needed in bagpack, so need to buy the qty set
			shopList[siegeItemId] = tonumber(mintoget)
		end
		
	end
	
	local shoppinginfo = {shopList, alreadygot, goldList, neededList}
	return shoppinginfo
	
end

-- Runs when you talk to any merchant
local function OnOpenStore()
	
	-- Only in Cyrodiil
	if IsInCyrodiil() then
	
		local shoppingInfo = GetShoppingList()
		local shoppingList = shoppingInfo[1]
		local alreadygot = shoppingInfo[2]
		local goldList = shoppingInfo[3]
		
		local alliancePointsBefore = GetAlliancePoints()
		local moneyBefore = GetCurrentMoney()
		local hasBuyed = false
		local apSpent = 0
		local goldSpent = 0
		local minAPtoKeep
		local minGoldtoKeep
		local bought = {}
		
		if db.mingoldtokeep then
			minGoldtoKeep = tonumber(db.mingoldtokeep) or 0
		else
			minGoldtoKeep = 0
		end
		
		if db.minaptokeep then
			minAPtoKeep = tonumber(db.minaptokeep) or 0
		else
			minAPtoKeep = 0
		end
		
		-- Whole items sold at NPC
		for index = 1, GetNumStoreItems() do
			-- Some NPC can sold a stack of things and some npc can sold same items with differents currencies or prices
			local _, itemName, stack, price, _, _, _, _, _, currency, currencyQty = GetStoreEntryInfo(index)
			
			-- Get ID of item sold by NPC
			local itemId = GetItemLinkItemId(GetStoreItemLink(index))
			
			-- If NPC get the item needed, sell its with AP and need to buy it
			-- Even if we buy using CURT_MONEY (and CURT_NONE == 1), the currency returned is always CURT_NONE for money.
			if shoppingList[itemId] and stack > 0 and ((currency == CURT_ALLIANCE_POINTS and not goldList[itemId]) or (goldList[itemId] and currency == CURT_NONE)) and not bought[itemId] then
				-- Some NPC cannot sold infinite items, they got a max, GetStoreEntryMaxBuyable also returns the min we can buy depending on our current money
				local quantity = zo_min(shoppingList[itemId], GetStoreEntryMaxBuyable(index))
				
				-- Enought AP to buy something
				if quantity > 0 then
					
					-- Warning to say that we can buy the whole qty
					if quantity < shoppingList[itemId] then
						CHAT_SYSTEM:AddMessage("Can buy only x" .. quantity .. " (instead of " .. shoppingList[itemId] .. ") " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", price = " .. (currencyQty * shoppingList[itemId]) .. " - Not enought AP ")
					end
					
					local freeSlots = GetNumBagFreeSlots(BAG_BACKPACK)
					-- Check if bagpack can countain items (maybe bagpack is full?)
					if freeSlots == 0 then
						-- For consomables, we can buy even if bag if full, TODO, handle maxsize / stack (should not occurs often)
						if IsSiegeConsumable(itemId) and alreadygot[itemId] then
							-- I can buy
							-- Check if the cost is more expensive to the min ap to keep
							if goldList[itemId] then
								if (GetCurrentMoney() - (price * quantity)) > minGoldtoKeep then
									BuyStoreItem(index, quantity)
									hasBuyed = true
									goldSpent = goldSpent + (price * quantity)
									CHAT_SYSTEM:AddMessage("Bought x" .. quantity .. " " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", price = " .. (price * quantity) .. " - WARNING : Bag is full !")
								end
							elseif (GetAlliancePoints() - (currencyQty * quantity)) > minAPtoKeep then
								BuyStoreItem(index, quantity)
								hasBuyed = true
								apSpent = apSpent + (currencyQty * quantity)
								CHAT_SYSTEM:AddMessage("Bought x" .. quantity .. " " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", price = " .. (currencyQty * quantity) .. " - WARNING : Bag is full !")
							else
								-- Reached min AP to keep
								CHAT_SYSTEM:AddMessage("Cannot buy x" .. quantity .. " " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", price = " .. (currencyQty * quantity) .. " - Reached min " .. minAPtoKeep .." AP to keep")
							end
						else
							-- I can't buy, no space left
							CHAT_SYSTEM:AddMessage("Cannot buy x" .. quantity .. " " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", price = " .. (currencyQty * quantity) .. " - Bag is full")
						end
					else
						
						-- For siege items, qty = numberofslots needed
						if not IsSiegeConsumable(itemId) then
							-- Bagpack contains freeslots, but maybe we can't buy the qty needed ?
							if freeSlots < quantity then
								-- Only buy the qty my bag can handle
								quantity = freeSlots
							end
						end
						
						-- Check if the cost is more expensive to the min ap to keep
						if goldList[itemId] then
							if (GetCurrentMoney() - (price * quantity)) > minGoldtoKeep then
								hasBuyed = true
								BuyStoreItem(index, quantity)
								goldSpent = goldSpent + (price * quantity)
								CHAT_SYSTEM:AddMessage("Bought x" .. quantity .. " " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", Gold = " .. (price * quantity) )
								bought[itemId] = true
							end
						elseif (GetAlliancePoints() - (currencyQty * quantity)) > minAPtoKeep then
							hasBuyed = true
							BuyStoreItem(index, quantity)
							apSpent = apSpent + (currencyQty * quantity)
							bought[itemId] = true
							CHAT_SYSTEM:AddMessage("Bought x" .. quantity .. " " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", AP = " .. (currencyQty * quantity) )
						else
							-- Reached min AP to keep
							bought[itemId] = true
							CHAT_SYSTEM:AddMessage("Cannot buy " .. quantity .. " " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", price = " .. (currencyQty * quantity) .. " - Reached min " .. minAPtoKeep .." AP to keep")
						end
					end
					
				-- Not enought AP
				else
					CHAT_SYSTEM:AddMessage("Cannot buy " .. ZO_LinkHandler_CreateLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemName), LINK_STYLE_DEFAULT, ITEM_LINK_TYPE, itemId .. ":1:0:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0:0") .. ", price = " .. currencyQty .. " - Not enought AP/Gold")
				end
								
			end
		end
		
		if hasBuyed then
			if apSpent > 0 then
				CHAT_SYSTEM:AddMessage("Spent " .. ZO_CommaDelimitNumber(apSpent) .. " AP (Before: " .. ZO_CommaDelimitNumber(alliancePointsBefore) .. " / After: " .. ZO_CommaDelimitNumber(alliancePointsBefore - apSpent) .. ")")
			end
			if goldSpent > 0 then
				CHAT_SYSTEM:AddMessage("Spent " .. ZO_CommaDelimitNumber(goldSpent) .. " Gold (Before: " .. ZO_CommaDelimitNumber(moneyBefore) .. " / After: " .. ZO_CommaDelimitNumber(moneyBefore - goldSpent) .. ")")
			end
		end
		
	end
	
end

local function MoveItemsAtBank()

	local tinyBagCache = {
		[BAG_BACKPACK] = {},
		[BAG_BANK] = {},
	}
	
	local tinyBagCacheFirstSlot = {
		[BAG_BACKPACK] = 0,
		[BAG_BANK] = 0,
	}
	
	local freeSlots = {
		[BAG_BACKPACK] = 0,
		[BAG_BANK] = 0,
	}
	
	local function FindEmptySlotInBag(bagId)
		for slotIndex = tinyBagCacheFirstSlot[bagId], (GetBagSize(bagId) - 1) do
			if not SHARED_INVENTORY.bagCache[bagId][slotIndex] and not tinyBagCache[bagId][slotIndex] then
				tinyBagCache[bagId][slotIndex] = true
				tinyBagCacheFirstSlot[bagId] = slotIndex + 1
				freeSlots[bagId] = freeSlots[bagId] - 1
				return slotIndex
			end
		end
		return nil
	end
	
	local function MoveItemInSlot(bagIdFrom, slotIdFrom, bagIdTo, slotIdTo, qtyToMove)
		if IsProtectedFunction("RequestMoveItem") then
			CallSecureProtected("RequestMoveItem", bagIdFrom, slotIdFrom, bagIdTo, slotIdTo, qtyToMove)
		else
			RequestMoveItem(bagIdFrom, slotIdFrom, bagIdTo, slotIdTo, qtyToMove)
		end
		
		-- to check
		return true
		
	end

	if GetInteractionType() == INTERACTION_BANK then
	
		local qtyMoved = {}
	
		if IsInCyrodiil() then
		
			local shoppingInfo = GetShoppingList()
			
			if db.CyroBank == 2 then
				
				freeSlots[BAG_BACKPACK] = GetNumBagFreeSlots(BAG_BACKPACK)
				
				if freeSlots[BAG_BACKPACK] > 0 then
				
					local neededList = shoppingInfo[4]
					
					for index, slotData in pairs(SHARED_INVENTORY.bagCache[BAG_BANK]) do
					
						local itemLink = GetItemLink(BAG_BANK, slotData.slotIndex)
						local itemId = GetItemLinkItemId(itemLink)
						
						if neededList[itemId] then
						
							if not qtyMoved[itemId] then
								qtyMoved[itemId] = {}
								qtyMoved[itemId].q = 0
								qtyMoved[itemId].n = slotData.name
							end
							
							if freeSlots[BAG_BACKPACK] > 0 then
								local destSlot = FindEmptySlotInBag(BAG_BACKPACK)
								qtyMoved[itemId].q = qtyMoved[itemId].q + slotData.stackCount
								MoveItemInSlot(BAG_BANK, slotData.slotIndex, BAG_BACKPACK, destSlot, slotData.stackCount)
							else
								break
							end
						end
					end
					
				end
				
			elseif db.CyroBank == 3 then
				
				freeSlots[BAG_BACKPACK] = GetNumBagFreeSlots(BAG_BACKPACK)
				
				if freeSlots[BAG_BACKPACK] > 0 then

					local neededList = shoppingInfo[1]
					
					for index, slotData in pairs(SHARED_INVENTORY.bagCache[BAG_BANK]) do
						
						local itemLink = GetItemLink(BAG_BANK, slotData.slotIndex)
						local itemId = GetItemLinkItemId(itemLink)
						
						if neededList[itemId] and neededList[itemId] > 0 then
							
							if not qtyMoved[itemId] then
								qtyMoved[itemId] = {}
								qtyMoved[itemId].q = 0
								qtyMoved[itemId].n = slotData.name
							end
							
							if freeSlots[BAG_BACKPACK] > 0 and qtyMoved[itemId].q < neededList[itemId] then
								local destSlot = FindEmptySlotInBag(BAG_BACKPACK)
								local qtyToMove = math.min(slotData.stackCount, neededList[itemId])
								qtyMoved[itemId].q = qtyMoved[itemId].q + qtyToMove
								MoveItemInSlot(BAG_BANK, slotData.slotIndex, BAG_BACKPACK, destSlot, qtyToMove)
							else
								break
							end
						end
					end
					
				end
			end
			
			for itemId, data in pairs(qtyMoved) do
				CHAT_SYSTEM:AddMessage("Moved x" .. data.q .. " " .. zo_strformat(SI_TOOLTIP_ITEM_NAME, data.n) .. " in bag")
			end
			
		else
			
			local shoppingInfo = GetShoppingList()
			
			if db.PveBank == 2 then
				
				freeSlots[BAG_BANK] = GetNumBagFreeSlots(BAG_BANK)
				
				if freeSlots[BAG_BANK] > 0 then
				
					for index, slotData in pairs(SHARED_INVENTORY.bagCache[BAG_BACKPACK]) do
					
						local itemLink = GetItemLink(BAG_BACKPACK, slotData.slotIndex)
						local itemId = GetItemLinkItemId(itemLink)
						
						if slotData.itemType == ITEMTYPE_AVA_REPAIR or slotData.itemType == ITEMTYPE_SIEGE then
							
							if not qtyMoved[itemId] then
								qtyMoved[itemId] = {}
								qtyMoved[itemId].q = 0
								qtyMoved[itemId].n = slotData.name
							end
							
							if freeSlots[BAG_BANK] > 0 then
								local destSlot = FindEmptySlotInBag(BAG_BANK)
								qtyMoved[itemId].q = qtyMoved[itemId].q + slotData.stackCount
								MoveItemInSlot(BAG_BACKPACK, slotData.slotIndex, BAG_BANK, destSlot, slotData.stackCount)
							else
								break
							end
						end
					end
					
				end
				
			elseif db.PveBank == 3 then
				
				freeSlots[BAG_BANK] = GetNumBagFreeSlots(BAG_BANK)
				
				if freeSlots[BAG_BANK] > 0 then

					local alreadyGot = shoppingInfo[2]
					local neededList = shoppingInfo[4]
						
					for index, slotData in pairs(SHARED_INVENTORY.bagCache[BAG_BACKPACK]) do
						
						local itemLink = GetItemLink(BAG_BACKPACK, slotData.slotIndex)
						local itemId = GetItemLinkItemId(itemLink)
						
						if slotData.itemType == ITEMTYPE_AVA_REPAIR or slotData.itemType == ITEMTYPE_SIEGE then
							
							if not qtyMoved[itemId] then
								qtyMoved[itemId] = {}
								qtyMoved[itemId].q = 0
								qtyMoved[itemId].n = slotData.name
							end
							
							if neededList[itemId] == 0 then
							
								if freeSlots[BAG_BANK] > 0 then
									local destSlot = FindEmptySlotInBag(BAG_BANK)
									qtyMoved[itemId].q = qtyMoved[itemId].q + slotData.stackCount
									MoveItemInSlot(BAG_BACKPACK, slotData.slotIndex, BAG_BANK, destSlot, slotData.stackCount)
								else
									break
								end
							
							else
							
								if not qtyToPush[itemId] then qtyToPush[itemId] = alreadyGot[itemId] - neededList[itemId]  end
								
								if qtyToPush[itemId] > 0 then
									
									if freeSlots[BAG_BANK] > 0 then
									
										local destSlot = FindEmptySlotInBag(BAG_BANK)
										local qtyToMove = math.min(slotData.stackCount, qtyToPush[itemId])
										qtyToPush[itemId] = qtyToPush[itemId] - qtyToMove
										qtyMoved[itemId].q = qtyMoved[itemId].q + qtyToMove
										MoveItemInSlot(BAG_BACKPACK, slotData.slotIndex, BAG_BANK, destSlot, qtyToMove)
									else
										break
									end
									
								end
								
							end
						end
					end
				end
			end
		
			for itemId, data in pairs(qtyMoved) do
				CHAT_SYSTEM:AddMessage("Moved x" .. data.q .. " " .. zo_strformat(SI_TOOLTIP_ITEM_NAME, data.n) .. " in bank")
			end
		
		end
	
	end

end

local function OnOpenBank()
	if db.autoBank then
		zo_callLater(MoveItemsAtBank, 500)
	end
end

-- Init settings with LAM
local function OnAddonLoaded(_, addon)

	if addon == ADDON_NAME then
	
		-- Register librairies
		local LAM = LibStub("LibAddonMenu-2.0")
		
		-- Initialize defaut var for SavedVariables
		local defaults = {
			shoppinglist = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
			goldlist = {false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false},
			CyroBankCyroBank == 1,
			PveBank = 1,
			autoBank = false,
		}
		
		-- Fetch the saved variables
		db = ZO_SavedVars:NewAccountWide('CYROSIEGEOPTS', 1, nil, defaults)
		
		-- Create control panel
		local panelData = {
			type = "panel",
			name = ADDON_NAME,
			displayName = ZO_HIGHLIGHT_TEXT:Colorize(ADDON_NAME),
			author = ADDON_AUTHOR,
			version = ADDON_VERSION,
			slashCommand = "/CyroSiege",
			registerForRefresh = true,
			registerForDefaults = true,
			website = ADDON_WEBSITE,
		}

		-- Start adding elements to control panel
		local optionsTable = {
			-- Minimum Stones
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME2))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME2))),
				getFunc = function() return db.shoppinglist[2] end,
				setFunc = function(newValue) db.shoppinglist[2]= newValue end,
				width = "half",
				default = 0,
			},
			-- Buy in gold
			{
				type = "checkbox",
				name = GetString(CYROSEGE_BUYINGOLD),
				tooltip = GetString(CYROSEGE_BUYINGOLD_DESC),
				getFunc = function() return db.goldlist[2] end,
				setFunc = function(newValue) db.goldlist[2] = newValue end,
				width = "half",
				default = false,
			},
			-- Minimum Wood
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME3))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME3))),
				getFunc = function() return db.shoppinglist[3] end,
				setFunc = function(newValue) db.shoppinglist[3] = newValue end,
				width = "half",
				default = 0,
			},
			-- Buy in gold
			{
				type = "checkbox",
				name = GetString(CYROSEGE_BUYINGOLD),
				tooltip = GetString(CYROSEGE_BUYINGOLD_DESC),
				getFunc = function() return db.goldlist[3] end,
				setFunc = function(newValue) db.goldlist[3]= newValue end,
				width = "half",
				default = false,
			},
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME7))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME7))),
				getFunc = function() return db.shoppinglist[7] end,
				setFunc = function(newValue) db.shoppinglist[7] = newValue end,
				width = "half",
				default = 0,
			},
			-- Buy in gold
			{
				type = "checkbox",
				name = GetString(CYROSEGE_BUYINGOLD),
				tooltip = GetString(CYROSEGE_BUYINGOLD_DESC),
				getFunc = function() return db.goldlist[7] end,
				setFunc = function(newValue) db.goldlist[7] = newValue end,
				width = "half",
				default = false,
			},
			-- Minimum Naphts Trebs
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME9))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME9))),
				getFunc = function() return db.shoppinglist[9] end,
				setFunc = function(newValue) db.shoppinglist[9] = newValue end,
				width = "half",
				default = 0,
			},
			-- Buy in gold
			{
				type = "checkbox",
				name = GetString(CYROSEGE_BUYINGOLD),
				tooltip = GetString(CYROSEGE_BUYINGOLD_DESC),
				getFunc = function() return db.goldlist[9] end,
				setFunc = function(newValue) db.goldlist[9] = newValue end,
				width = "half",
				default = false,
			},
			{type = "texture", image="EsoUI/Art/Miscellaneous/horizontalDivider.dds", imageWidth=510, imageHeight=4},
			-- Minimum Oil
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME1))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME1))),
				getFunc = function() return db.shoppinglist[1] end,
				setFunc = function(newValue) db.shoppinglist[1] = newValue end,
				width = "full",
				default = 0,
			},			
			-- Minimum Tents
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME4))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME4))),
				getFunc = function() return db.shoppinglist[4] end,
				setFunc = function(newValue) db.shoppinglist[4]= newValue end,
				width = "full",
				default = 0,
			},
			-- Minimum Rammer
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME5))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME5))),
				getFunc = function() return db.shoppinglist[5] end,
				setFunc = function(newValue) db.shoppinglist[5] = newValue end,
				width = "full",
				default = 0,
			},
			{type = "texture", image="EsoUI/Art/Miscellaneous/horizontalDivider.dds", imageWidth=510, imageHeight=4},
			-- Minimum Std Balistas
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME6))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME6))),
				getFunc = function() return db.shoppinglist[6] end,
				setFunc = function(newValue) db.shoppinglist[6] = newValue end,
				width = "full",
				default = 0,
			},

			-- Minimum Air Balistas
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME8))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME8))),
				getFunc = function() return db.shoppinglist[8] end,
				setFunc = function(newValue) db.shoppinglist[8] = newValue end,
				width = "full",
				default = 0,
			},
			{type = "texture", image="EsoUI/Art/Miscellaneous/horizontalDivider.dds", imageWidth=510, imageHeight=4},
			-- Minimum Ice Trebs
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME10))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME10))),
				getFunc = function() return db.shoppinglist[10] end,
				setFunc = function(newValue) db.shoppinglist[10] = newValue end,
				width = "full",
				default = 0,
			},
			-- Minimum Stone Trebs
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME11))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME11))),
				getFunc = function() return db.shoppinglist[11] end,
				setFunc = function(newValue) db.shoppinglist[11] = newValue end,
				width = "full",
				default = 0,
			},
			{type = "texture", image="EsoUI/Art/Miscellaneous/horizontalDivider.dds", imageWidth=510, imageHeight=4},
			-- Minimum Meatbag Catapults
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME12))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME13))),
				getFunc = function() return db.shoppinglist[12] end,
				setFunc = function(newValue) db.shoppinglist[12] = newValue end,
				width = "full",
				default = 0,
			},
			-- Minimum Oil Catapults
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME13))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME13))),
				getFunc = function() return db.shoppinglist[13] end,
				setFunc = function(newValue) db.shoppinglist[13] = newValue end,
				width = "full",
				default = 0,
			},
			-- Minimum Scattershot Catapults
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME14))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME14))),
				getFunc = function() return db.shoppinglist[14] end,
				setFunc = function(newValue) db.shoppinglist[14] = newValue end,
				width = "full",
				default = 0,
			},
			{type = "texture", image="EsoUI/Art/Miscellaneous/horizontalDivider.dds", imageWidth=510, imageHeight=4},
			-- Minimum Siege Repair Kit
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME15))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME15))),
				getFunc = function() return db.shoppinglist[15] end,
				setFunc = function(newValue) db.shoppinglist[15] = newValue end,
				width = "full",
				default = 0,
			},
			-- Minimum Grand Soul Gems
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME16))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME16))),
				getFunc = function() return db.shoppinglist[16] end,
				setFunc = function(newValue) db.shoppinglist[16] = newValue end,
				width = "full",
				default = 0,
			},
			{type = "texture", image="EsoUI/Art/Miscellaneous/horizontalDivider.dds", imageWidth=510, imageHeight=4},
			-- Stamina Pot
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME17))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME17))),
				getFunc = function() return db.shoppinglist[17] end,
				setFunc = function(newValue) db.shoppinglist[17] = newValue end,
				width = "full",
				default = 0,
			},
			-- Health Pot
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME18))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME18))),
				getFunc = function() return db.shoppinglist[18] end,
				setFunc = function(newValue) db.shoppinglist[18] = newValue end,
				width = "full",
				default = 0,
			},
			-- Magicka Pot
			{
				type = "editbox",
				name = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME19))),
				tooltip = zo_strformat(SI_TOOLTIP_ITEM_NAME, factionItemsNames(GetString(CYROSEGE_ITEMNAME19))),
				getFunc = function() return db.shoppinglist[19] end,
				setFunc = function(newValue) db.shoppinglist[19] = newValue end,
				width = "full",
				default = 0,
			},
			{type = "texture", image="EsoUI/Art/Miscellaneous/horizontalDivider.dds", imageWidth=510, imageHeight=4},
			-- Minimum AP to Keep
			{
				type = "editbox",
				name = GetString(CYROSEGE_MINAPTOKEEP),
				tooltip = GetString(CYROSEGE_MINAPTOKEEP_DESC),
				getFunc = function() return db.minaptokeep end,
				setFunc = function(newValue) db.minaptokeep = newValue end,
				width = "full",
				default = 0,
			},
			-- Minimum Gold to Keep
			{
				type = "editbox",
				name = GetString(CYROSEGE_MINGOLDTOKEEP),
				tooltip = GetString(CYROSEGE_MINGOLDTOKEEP_DESC),
				getFunc = function() return db.mingoldtokeep end,
				setFunc = function(newValue) db.mingoldtokeep = newValue end,
				width = "full",
				default = 0,
			},
			{type = "texture", image="EsoUI/Art/Miscellaneous/horizontalDivider.dds", imageWidth=510, imageHeight=4},
			{
				type = "dropdown",
				name = GetString(CYROSEGE_BANK_INCYRO),
				tooltip = GetString(CYROSEGE_BANK_INCYRO_DESC),
				choices = {GetString(CYROSIEGE_NOTHING), GetString(CYROSIEGE_WITHDRAW_ALL), GetString(CYROSIEGE_ONLY_MISSING)},
				getFunc = function()
					if db.CyroBank == 1 then
						return GetString(CYROSIEGE_NOTHING)
					elseif db.CyroBank == 2 then
						return GetString(CYROSIEGE_WITHDRAW_ALL)
					elseif db.CyroBank == 3 then
						return GetString(CYROSIEGE_ONLY_MISSING)
					else
						-- When user click on LAM reinit button
						return GetString(CYROSIEGE_NOTHING)
					end
				end,
				setFunc = function(choice)
					if choice == GetString(CYROSIEGE_NOTHING) then
						db.CyroBank = 1
					elseif choice == GetString(CYROSIEGE_WITHDRAW_ALL) then
						db.CyroBank = 2
					elseif choice == GetString(CYROSIEGE_ONLY_MISSING) then
						db.CyroBank = 3
					else
						db.CyroBank = 1
					end
				end,
				default = 1,
			},
			{
				type = "dropdown",
				name = GetString(CYROSEGE_BANK_INPVE),
				tooltip = GetString(CYROSEGE_BANK_INPVE_DESC),
				choices = {GetString(CYROSIEGE_NOTHING), GetString(CYROSIEGE_DEPOSIT_SIEGE), GetString(CYROSIEGE_DEPOSIT_SURPLUS)},
				getFunc = function()
					if db.PveBank == 1 then
						return GetString(CYROSIEGE_NOTHING)
					elseif db.PveBank == 2 then
						return GetString(CYROSIEGE_DEPOSIT_SIEGE)
					elseif db.PveBank == 3 then
						return GetString(CYROSIEGE_DEPOSIT_SURPLUS)
					else
						return GetString(CYROSIEGE_NOTHING)
					end
				end,
				setFunc = function(choice)
					if choice == GetString(CYROSIEGE_NOTHING) then
						db.PveBank = 1
					elseif choice == GetString(CYROSIEGE_DEPOSIT_SIEGE) then
						db.PveBank = 2
					elseif choice == GetString(CYROSIEGE_DEPOSIT_SURPLUS) then
						db.PveBank = 3
					else
						db.PveBank = 1
					end			
				end,
				default = 1,
			},
			{
				type = "checkbox",
				name = GetString(CYROSEGE_AUTOBANK),
				tooltip = GetString(CYROSEGE_AUTOBANK_DESC),
				getFunc = function() return db.autoBank end,
				setFunc = function(newValue) db.autoBank = newValue end,
				width = "full",
				default = 0,
			},
		}
		
		LAM:RegisterAddonPanel("CyroSiegeOptions", panelData)
		LAM:RegisterOptionControls("CyroSiegeOptions", optionsTable)
		
		ZO_CreateStringId("SI_BINDING_NAME_CS_TRANSFER", GetString(CYROSIEGE_TRANSFER_ITEMS))
		
		-- When talking to an NPC
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_OPEN_STORE, OnOpenStore)
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_OPEN_BANK, OnOpenBank)
		
		EVENT_MANAGER:UnregisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED)
	
	end
	
end

-- Called by Keybindings
function CyroSiege_MoveItemsAtBank()
	MoveItemsAtBank()
end

-- Set addon to load when player enters the world.
EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED, OnAddonLoaded)